/*	Autor: Adrian Fiergolski EIKPE
	
	  Program prezentujcy przyklad uycia Multimapy
Opis: Multimapa to posortowany kontener asocjacyjny, czyli zbiornik o zmiennej dugoci gromadzcy dane,
	  ktre mona dodawa i usuwa. Jest ona zbiornikiem parowym, a wic jej elementami s pary wartoci: 
	  klucz i dana. Pierwszej wartoci typu key_type, bdcej kluczem multimapy, nie mona zmienia,
	  natomiast druga warto danej jest przypisywalna (np.(*i).second=2). Elementw multimapy nie mona
	  dodawa na konkretn pozycje, poniewa ich kolejno ustalana jest wg danej klucz. Multimapa nie
	  jest unikalnym kontenerem asocjacyjnym, a wic dwa elementy mog posiada ten sam klucz.
*/

	 //Nagwek z deklaracj Multimap'y
#include <map>	
	 //Nagwek z deklaracj ostream_iterator
#include <iterator> 

#include <string>
#include <iostream>

using namespace std;

/*	  Alias multimapy przechowujcej nazwy miesica wedug ich dugoci. Kluczem jest int - dugo miesica 
	  w dniach; za przechowywan wartoci string - nazwa miesica. Trzeci parametr okrela obiekt funkcyjny
	  porwnujcy dwa klucze w celu nadania porzdku w kontenerze. Domylnie jest to funktor less<T>, w przy-
	  kadzie zostanie zastosowany greater<T> - miesice bd przechowywane od najduszego do najkrtszego.
	  Multimapa bdzie korzysta z domylnego allocatora (czwarty domylny argument).
*/
typedef multimap<int, string, greater<int> > months_type;


/*	   Szablony z przecionymi operatorami << naley doczy do przestrzeni nazw std, w przeciwnym razie nie
	   byyby wykorzystywane z powodu "Koenig Lookup" => www.gotw.ca/gotw/030.htm
*/
namespace std
{
/*	   Szablon przecienia operatora <<, ktry posuy do wypisywania na standardowym wyjciu nazwy miesica
	   i jego dugo, na podstawie pair<T,T>*/
	template <class First, class Second>   
		ostream& operator<<(ostream& out, const pair<First,Second>& p)
		{
			cout << p.second << " ma " << p.first << " dni";
			return out;
		}

/*	   Przecienie operatora <<, ktrre posuy do wyetlania na standardowym wyjciu zawartoci multimapy.*/
	ostream& operator<<(ostream& out, months_type& m)
	{
/*	   Wykorzystanie copy z biblioteki stl,ktry kopiuje pewien zakres multimapy do ostram_iterator.
	   ostream_iterator jest iteratorem wyjcia (umoliwia tylko zapis wartoci do danego skadnika 
	   odczyt jest niemoliwy). Jego szablon wymaga zdefiniowania typu przechowywanych elementw 
	   w strumieniu. Konstruktor ostream_iterator wywoywane jest z parametrami, ktre definiuj strumie
	   oraz char* dodawany na kocu kadego elementu.
	   Samo poruszanie si po multimapie zrealizowano, z wykorzystaniem jej iteratorw (m.begin(),m.end).
*/
		copy(m.begin(),m.end(), ostream_iterator<months_type::value_type>(cout,"\n"));
		return out;
	}
}

int main(void)
{
/*   Tworzenie multimapy miesicy*/
    months_type months;

/*   Alias typu obiektu, pair<const key_type, data_type>, magazynowanego w multimapie*/
    typedef months_type::value_type value_type;

/*   Dodawanie miesicy do multimapy: pierwszy w parze jest klucz, nastpnie sama warto elementu*/
    months.insert(value_type(31, string("Styczen")));
    months.insert(value_type(28, string("Luty")));
    months.insert(value_type(31, string("Marzec")));
    months.insert(value_type(30, string("Kwiecien")));
    months.insert(value_type(31, string("Maj")));
    months.insert(value_type(30, string("Czerwiec")));
    months.insert(value_type(31, string("Lipiec")));
    months.insert(value_type(31, string("Sierpien")));
    months.insert(value_type(30, string("Wrzesien")));
    months.insert(value_type(31, string("Pazdziernik")));
    months.insert(value_type(30, string("Listopad")));
    months.insert(value_type(31, string("Grudzien")));


/*   Wywietlenie zawartoci multimapy - naley zauwayc,e miesice bd one posortowanie zgodnie z za-
     danym wczeniej funktorem - od najduszego do najkrtszego. Metoda size() na rzecz multimapy zwraca
     jej rozmiar*/
    cout << "Wszystkie miesiace roku ("<< months.size()<<"):" << endl << months << endl;

/*   Wyszukanie miesicy z 30 dniami (wyszukiwanie elementw o zadanym kluczu i zwrcenie pary iteratorw
     okrelajcych zakres wynikw wyszukiwania)*/
    pair<months_type::iterator,months_type::iterator> p = months.equal_range(30);

/*   Wywietlenie znalezionych miesicy z wykorzystaniem copy. Metoda count() zwraca ilo elementw 
     o zadanym kluczu */
    cout << endl << "Miesiace z 30 dniami ("<<months.count(30)<<"):" << endl;
    copy(p.first,p.second, ostream_iterator<months_type::value_type>(cout,"\n"));

/*   Usuniecie czci elementw z multimapy okrelonych przez iteratory z wykorzystaniem metody erase()
     i wyswietlenie zawartoci multimapy*/ 	 
    months.erase(p.first,p.second);
    cout<<"\nRok pozbawiony miesiecy o 30 dniach ("<<months.size()<<"):\n"<<months<<endl;

    return 0;
}

/*   Wydruk programu:
		Wszystkie miesiace roku (12):
		Styczen ma 31 dni
		Marzec ma 31 dni
		Maj ma 31 dni
		Lipiec ma 31 dni
		Sierpien ma 31 dni
		Pazdziernik ma 31 dni
		Grudzien ma 31 dni
		Kwiecien ma 30 dni
		Czerwiec ma 30 dni
		Wrzesien ma 30 dni
		Listopad ma 30 dni
		Luty ma 28 dni


		Miesiace z 30 dniami (4):
		Kwiecien ma 30 dni
		Czerwiec ma 30 dni
		Wrzesien ma 30 dni
		Listopad ma 30 dni

		Rok pozbawiony miesiecy o 30 dniach (8):
		Styczen ma 31 dni
		Marzec ma 31 dni
		Maj ma 31 dni
		Lipiec ma 31 dni
		Sierpien ma 31 dni
		Pazdziernik ma 31 dni
		Grudzien ma 31 dni
		Luty ma 28 dni
*/


